/*********************************************************************

   Group   - Apocalypse Games

   Project - Dominate

   Target  - big.exe
   File    - big.cpp
   Author  - Jeff Weston

   Purpose - Source Code for BIG file utility program

**********************************************************************

RCS Information :

   $Id: big.cpp 1.9 1997/09/02 07:36:51 Jeff_Weston Exp Jeff_Weston $

   $Log: big.cpp $
   Revision 1.9  1997/09/02 07:36:51  Jeff_Weston
   - Added an option to extract all the files from a BIG file.

   Revision 1.8  1997/08/23 01:07:38  Jeff_Weston
   - Removed all delay code.

   Revision 1.7  1997/08/22 01:42:18  Jeff_Weston
   - Changed BIG file format : file entry list is terminated by a 0x0000.

   Revision 1.6  1997/08/21 20:09:18  Jeff_Weston
   - Added code to create BIG file.
   - Support for already existing BIG file is not present yet.
   - Support for adding or deleting files is not present yet.

   Revision 1.5  1997/08/21 07:10:39  Jeff_Weston
   - Fixed page fault bug.

   Revision 1.4  1997/08/17 23:41:48  Jeff_Weston
   - Added additional command line argument checking.

   Revision 1.3  1997/08/10 21:39:44  Jeff_Weston
   - Added code to sort file list and display length of the name string.

   Revision 1.2  1997/08/01 00:37:56  Jeff_Weston
   - Added code to get a list of files, based on command line arguments.

   Revision 1.1  1997/07/31 00:08:19  Jeff_Weston
   Initial revision


*********************************************************************/
#include "big.h"

#pragma argsused

FILE_ENTRY::FILE_ENTRY ()
{
  strFileName = "";
  strFilePath = "";
  iFileSize   = 0;
}

FILE_ENTRY::FILE_ENTRY (const FILE_ENTRY& feTemp)
{
  strFileName = feTemp.strFileName;
  strFilePath = feTemp.strFilePath;
  iFileSize   = feTemp.iFileSize;
}

int operator <  (const FILE_ENTRY& left, const FILE_ENTRY& right)
{
  return (left.strFileName <  right.strFileName);
}

int operator >  (const FILE_ENTRY& left, const FILE_ENTRY& right)
{
  return (left.strFileName >  right.strFileName);
}

int operator <= (const FILE_ENTRY& left, const FILE_ENTRY& right)
{
  return (left.strFileName <= right.strFileName);
}

int operator >= (const FILE_ENTRY& left, const FILE_ENTRY& right)
{
  return (left.strFileName >= right.strFileName);
}

int operator == (const FILE_ENTRY& left, const FILE_ENTRY& right)
{
  return (left.strFileName == right.strFileName);
}

int operator != (const FILE_ENTRY& left, const FILE_ENTRY& right)
{
  return (left.strFileName != right.strFileName);
}

int main (int argc, char* argv[])
{
  HANDLE                   hStdout, hFileSearch, hBigFile, hLoadFile;
  unsigned long            uiTemp, iBytesWritten, iSizeCount = 0;
  WIN32_FIND_DATA          FindFileData;
  BOOL                     bError;
  char                     buf[BUFFER], output[BUFFER], cTemp;
  int                      iOption, iDone, iNameLength;
  int                      iLoop, iTemp, iCount = 0;
  cstring                  strTemp, strPath;
  FILE_ENTRY               feTemp;
  orderedList<FILE_ENTRY>  FileList;
  listIterator<FILE_ENTRY> FileListIterator(FileList);

  hStdout = GetStdHandle (STD_OUTPUT_HANDLE);
  if (hStdout == INVALID_HANDLE_VALUE) return 1;

  WriteFile (hStdout, WELCOME1_STR, lstrlen (WELCOME1_STR),
             &iBytesWritten, NULL);
  WriteFile (hStdout, WELCOME2_STR, lstrlen (WELCOME2_STR),
             &iBytesWritten, NULL);
  WriteFile (hStdout, WELCOME3_STR, lstrlen (WELCOME3_STR),
             &iBytesWritten, NULL);

  if (argc < 3) return 1;

  if (strcmp (SWITCH_ADD, argv[1]) == 0)
  {
    WriteFile (hStdout, OPTION_ADD, lstrlen (OPTION_ADD),
               &iBytesWritten, NULL);
    iOption = 1;
  }

  if (strcmp (SWITCH_DEL, argv[1]) == 0)
  {
    WriteFile (hStdout, OPTION_DEL, lstrlen (OPTION_DEL),
               &iBytesWritten, NULL);
    iOption = 2;
  }

  if (strcmp (SWITCH_EXT, argv[1]) == 0)
  {
    WriteFile (hStdout, OPTION_EXT, lstrlen (OPTION_EXT),
               &iBytesWritten, NULL);
    iOption = 3;
  }

  wsprintf (buf, "Using File : %s\n\n", argv[2]);
  WriteFile (hStdout, buf, lstrlen (buf), &iBytesWritten, NULL);

  if (iOption < 3)
  // Create new big file from scratch.
  {
    if (argc < 4) return 1;

    for (iLoop = 3; iLoop < argc; iLoop++)
    {
      iTemp = 0;
      strTemp = "";
      strPath = "";
      while (argv[iLoop][iTemp] != '\0')
      {
        if (argv[iLoop][iTemp] == '\\')
        {
          strPath += strTemp;
          strPath += '\\';
          strTemp = "";
        }
        else strTemp += argv[iLoop][iTemp];
        iTemp++;
      }

      hFileSearch = FindFirstFile (argv[iLoop], &FindFileData);

      if (hFileSearch == INVALID_HANDLE_VALUE)
      {
        wsprintf (buf, "Error : %ld", GetLastError());
        WriteFile (hStdout, buf, lstrlen (buf), &iBytesWritten, NULL);
        return 1;
      }

      bError = TRUE;

      while (bError == TRUE)
      {
        if ((FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == 0)
        {
          strTemp = FindFileData.cFileName;
          toUpper (strTemp);

          feTemp.strFilePath = strPath;
          feTemp.strFileName = strTemp;
          feTemp.iFileSize = (FindFileData.nFileSizeHigh * MAXDWORD)
                           +  FindFileData.nFileSizeLow;

          if ((feTemp.iFileSize > 0) &&
              (FileList.includes (feTemp) == 0))
          {
            FileList.add (feTemp);
          }
        }
        bError = FindNextFile (hFileSearch, &FindFileData);
      }
      FindClose (hFileSearch);
    }

    hBigFile = CreateFile(argv[2], GENERIC_WRITE, 0, NULL,
                          CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, NULL);
    if (hBigFile == INVALID_HANDLE_VALUE) return 1;

// unsigned int - length of name string
// char *        - name string
// unsigned int  - offset to file from beginning of file
// unsigned int  - size of file
//
// to indicate end of file entry table, there is a final unsigned int of 0

    for (FileListIterator.init (); !FileListIterator; ++FileListIterator)
    {
      iSizeCount += sizeof (unsigned int);
      iSizeCount += FileListIterator ().strFileName.length ();
      iSizeCount += sizeof (unsigned int);
      iSizeCount += sizeof (unsigned int);
    }

    iSizeCount += sizeof (unsigned int);

    for (FileListIterator.init (); !FileListIterator; ++FileListIterator)
    {
      uiTemp = FileListIterator ().strFileName.length ();
      WriteFile (hBigFile, &uiTemp, sizeof (int), &iBytesWritten, NULL);

      WriteFile (hBigFile, FileListIterator ().strFileName,
                 FileListIterator ().strFileName.length (),
                 &iBytesWritten, NULL);

      WriteFile (hBigFile, &iSizeCount,
                 sizeof (int), &iBytesWritten, NULL);

      uiTemp = FileListIterator ().iFileSize;
      WriteFile (hBigFile, &uiTemp, sizeof (int), &iBytesWritten, NULL);

      iSizeCount += FileListIterator ().iFileSize;
    }

    // Indicate END of file entry table
    uiTemp = 0;
    WriteFile (hBigFile, &uiTemp, sizeof (int), &iBytesWritten, NULL);

    for (FileListIterator.init (); !FileListIterator; ++FileListIterator)
    {
      strTemp =  FileListIterator ().strFilePath;
      strTemp += FileListIterator ().strFileName;
      strcpy (output, strTemp);
      wsprintf (buf, "File : %-50s%10lu%10lu\n", output,
                FileListIterator ().iFileSize, ++iCount);
      WriteFile (hStdout, buf, lstrlen (buf), &iBytesWritten, NULL);

      hLoadFile = CreateFile(FileListIterator ().strFilePath +
                             FileListIterator ().strFileName,
                             GENERIC_READ, 0, NULL, OPEN_EXISTING,
                             FILE_ATTRIBUTE_ARCHIVE, NULL);
      if (hLoadFile == INVALID_HANDLE_VALUE) return 1;

      CopyFile (hLoadFile, hBigFile);

      CloseHandle (hLoadFile);
    }

    CloseHandle (hBigFile);
  }
  else
  // Extract all files from BIG file.
  {
    hBigFile = CreateFile(argv[2], GENERIC_READ, FILE_SHARE_READ, NULL,
                          OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL);
    if (hBigFile == INVALID_HANDLE_VALUE) return 1;

    ReadFile (hBigFile, &iNameLength, sizeof (int), &iBytesWritten, NULL);

    iDone = 0;
    while (iDone == 0)
    {
      if (iNameLength == 0) iDone = 1;
      else
      {
        strTemp = "";
        for (iLoop = 0; iLoop < iNameLength; iLoop++)
        {
          ReadFile (hBigFile, &cTemp, sizeof (char), &iBytesWritten, NULL);
          strTemp += cTemp;
        }
        ReadFile (hBigFile, &iNameLength, sizeof (int), &iBytesWritten, NULL);
        ReadFile (hBigFile, &iNameLength, sizeof (int), &iBytesWritten, NULL);

        BIG_ExtractRealFile (argv[2], strTemp);
        // This extracts the current file from the BIG file.

        ReadFile (hBigFile, &iNameLength, sizeof (int), &iBytesWritten, NULL);
      }
    }
    CloseHandle (hBigFile);
  }
  return 0;
}
