/*********************************************************************

   Group   - Apocalypse Games

   Project - Dominate

   Target  - bigutil.obj
   File    - bigutil.cpp
   Author  - Jeff Weston

   Purpose - Source Code for BIG file utilities

**********************************************************************

RCS Information :

   $Id: bigutil.cpp 1.5 1997/09/02 07:38:27 Jeff_Weston Exp Jeff_Weston $

   $Log: bigutil.cpp $
   Revision 1.5  1997/09/02 07:38:27  Jeff_Weston
   - Added code to extract a file from a BIG file using its real name.

   Revision 1.4  1997/09/01 02:41:20  Jeff_Weston
   - Added a function to support loading palettes from bitmaps in the BIG file.

   Revision 1.3  1997/08/22 23:46:09  Jeff_Weston
   - Added a function to determine if a file exists in the BIG file.

   Revision 1.2  1997/08/22 05:10:23  Jeff_Weston
   - Added ExtractFile function to extract files from the BIG file.
   - Added BIG_DDLoadBitmap function to load bitmaps from the BIG file.

   Revision 1.1  1997/08/21 20:27:35  Jeff_Weston
   Initial revision


*********************************************************************/

#include "bigutil.h"

int BIG_FileExists (const char* szBigFile, const char* szFileName)
{
  int                 iNameLength, iDone, iLoop, iFound;
  unsigned long int   iBytesWritten;
  char                cTemp;
  HANDLE              hBigFile;
  cstring             strFileName;

  iFound = 0;

  hBigFile = CreateFile(szBigFile, GENERIC_READ, 0, NULL,
                        OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL);

  ReadFile (hBigFile, &iNameLength, sizeof (int), &iBytesWritten, NULL);

  iDone = 0;
  while (iDone == 0)
  {
    if (iNameLength == 0) iDone = 1;
    else
    {
      strFileName = "";
      for (iLoop = 0; iLoop < iNameLength; iLoop++)
      {
        ReadFile (hBigFile, &cTemp, sizeof (char), &iBytesWritten, NULL);
        strFileName += cTemp;
      }
      ReadFile (hBigFile, &iNameLength, sizeof (int), &iBytesWritten, NULL);
      ReadFile (hBigFile, &iNameLength, sizeof (int), &iBytesWritten, NULL);

      if (lstrcmp (strFileName, szFileName) == 0)
      {
        iFound = 1;
        iDone = 1;
      }
      else
        ReadFile (hBigFile, &iNameLength, sizeof (int), &iBytesWritten, NULL);
    }
  }
  CloseHandle (hBigFile);
  return iFound;
}

void BIG_ExtractRealFile (const char* szBigFile, const char* szFileName)
{
  int                 iFileStart, iFileSize, iNameLength, iDone, iLoop;
  unsigned long int   iBytesWritten;
  char                cTemp;
  HANDLE              hBigFile, hTempFile;
  cstring             strFileName;

  hBigFile = CreateFile(szBigFile, GENERIC_READ, FILE_SHARE_READ, NULL,
                        OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL);

  ReadFile (hBigFile, &iNameLength, sizeof (int), &iBytesWritten, NULL);

  iDone = 0;
  while (iDone == 0)
  {
    if (iNameLength == 0) iDone = 1;
    else
    {
      strFileName = "";
      for (iLoop = 0; iLoop < iNameLength; iLoop++)
      {
        ReadFile (hBigFile, &cTemp, sizeof (char), &iBytesWritten, NULL);
        strFileName += cTemp;
      }
      ReadFile (hBigFile, &iFileStart, sizeof (int), &iBytesWritten, NULL);
      ReadFile (hBigFile, &iFileSize, sizeof (int), &iBytesWritten, NULL);

      if (lstrcmp (strFileName, szFileName) == 0)
      {
        hTempFile = CreateFile(strFileName, GENERIC_WRITE, 0, NULL,
                               CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, NULL);

        SetFilePointer (hBigFile, iFileStart, NULL, FILE_BEGIN);
        CopyFile (hBigFile, hTempFile, iFileSize);
        CloseHandle (hTempFile);
        iDone = 1;
      }
      else
        ReadFile (hBigFile, &iNameLength, sizeof (int), &iBytesWritten, NULL);
    }
  }
  CloseHandle (hBigFile);
}

cstring BIG_ExtractTempFile (const char* szBigFile, const char* szFileName)
{
  int                 iFileStart, iFileSize, iNameLength, iDone, iLoop;
  unsigned long int   iBytesWritten;
  char                szTempName [BUF_SIZE], cTemp;
  HANDLE              hBigFile, hTempFile;
  cstring             strFileName;

  hBigFile = CreateFile(szBigFile, GENERIC_READ, 0, NULL,
                        OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL);

  GetTempFileName (TEMP_PATH, TEMP_PREFIX, 0, szTempName);

  hTempFile = CreateFile(szTempName, GENERIC_WRITE, 0, NULL, OPEN_EXISTING,
                         FILE_ATTRIBUTE_ARCHIVE, NULL);

  ReadFile (hBigFile, &iNameLength, sizeof (int), &iBytesWritten, NULL);

  iDone = 0;
  while (iDone == 0)
  {
    if (iNameLength == 0) iDone = 1;
    else
    {
      strFileName = "";
      for (iLoop = 0; iLoop < iNameLength; iLoop++)
      {
        ReadFile (hBigFile, &cTemp, sizeof (char), &iBytesWritten, NULL);
        strFileName += cTemp;
      }
      ReadFile (hBigFile, &iFileStart, sizeof (int), &iBytesWritten, NULL);
      ReadFile (hBigFile, &iFileSize, sizeof (int), &iBytesWritten, NULL);

      if (lstrcmp (strFileName, szFileName) == 0)
      {
        SetFilePointer (hBigFile, iFileStart, NULL, FILE_BEGIN);
        CopyFile (hBigFile, hTempFile, iFileSize);
        CloseHandle (hTempFile);
        iDone = 1;
      }
      else
        ReadFile (hBigFile, &iNameLength, sizeof (int), &iBytesWritten, NULL);
    }
  }
  CloseHandle (hBigFile);
  strFileName = szTempName;
  return strFileName;
}

IDirectDrawSurface* BIG_DDLoadBitmap (const char* szBigFile, IDirectDraw* pdd,
                                      const char* szBitmap, int dx, int dy)

{
  cstring             strTempName;
  IDirectDrawSurface* ddsReturn;

  strTempName = BIG_ExtractTempFile (szBigFile, szBitmap);

  ddsReturn = DDLoadBitmap(pdd, strTempName, dx, dy);
  DeleteFile (strTempName);

  return ddsReturn;
}

IDirectDrawPalette* BIG_DDLoadPalette (const char* szBigFile, IDirectDraw* pdd,
                                       const char* szBitmap)
{
  cstring             strTempName;
  IDirectDrawPalette* ddsReturn;

  strTempName = BIG_ExtractTempFile (szBigFile, szBitmap);

  ddsReturn = DDLoadPalette(pdd, strTempName);
  DeleteFile (strTempName);

  return ddsReturn;
}
