//======================================================================
//  File:        string.h
//  Author:      Timothy A. Budd
//  Description: This file contains the interface of the string
//               class.
//
//  Copyright (c) 1992 by Timothy A. Budd.  All Rights Reserved.
//	may be reproduced for any non-commercial purpose
//======================================================================

#ifndef STRING_H
#define STRING_H

#include <iostream.h>


class substring;			// forward declaration



//----------------------------------------------------------------------
//	class string
//		better bounds checking, assignments which result
//		in copies, comparisons using relational operators,
//		high level operations such as substrings
//----------------------------------------------------------------------

class cstring
{
public:
    // constructors
    cstring();
    cstring(char);
    cstring(int);
    cstring(const char *);
    cstring(const cstring &);

    // destructor
    ~cstring();

    // assignment an catenation
    void         operator =  (const cstring & right);
    void         operator += (const cstring & right);

    // substring access
    substring    operator () (unsigned int start, unsigned int length);

    // input of an entire line of text
    istream &    getline(istream &);

    // number of characters in the string
    unsigned int length() const;

    // access to a single character
    char &       operator [](unsigned int) const;

    // compare - used in relational operators
    int          compare(const cstring &) const;

    // conversion to ordinary C string
    operator     const char *() const;

    // allow access by substring class
    friend class substring;
private:
    // data fields - character buffer and length
    unsigned int       bufferlength;
    char *             buffer;
};



//----------------------------------------------------------------------
//	class substring
//		facilitator class to permit access to
//		portions of a string
//----------------------------------------------------------------------

class substring
{
public:
    // constructors
    substring(cstring & base, int start, int length);
    substring(const substring & source);

    // assignment of string to substring
    void operator = (const cstring &) const;

    // conversion of substring to string
    operator cstring () const;

private:
    // data areas
    cstring &        base;
    const int       index;
    const int       count;
};



// Function Prototypes: relational operators

int    operator <  (const cstring &, const cstring &);
int    operator <= (const cstring &, const cstring &);
int    operator != (const cstring &, const cstring &);
int    operator == (const cstring &, const cstring &);
int    operator >= (const cstring &, const cstring &);
int    operator >  (const cstring &, const cstring &);



// Function Prototype: concatenate operator

cstring operator +  (const cstring &, const cstring &);



// Function Prototypes: character classification

int isUpperCase(char c);
int isLowerCase(char c);



// Function Prototypes: string conversions

void toUpper(cstring & word);
void toLower(cstring & word);


// Function Prototype: stream input

istream & operator >> (istream & in, cstring & str);



#endif
