/*********************************************************************

   Group   - Apocalypse Games

   Project - Dominate

   Target  - editor.exe
   File    - editor.cpp
   Author  - Jeff Weston

   Purpose - Source Code for Map Editor for Dominate

**********************************************************************

RCS Information :

   $Id: editor.cpp 1.17 1997/09/02 07:35:33 Jeff_Weston Exp Jeff_Weston $

   $Log: editor.cpp $
   Revision 1.17  1997/09/02 07:35:33  Jeff_Weston
   - The editor now uses the cursor from CURSOR.BMP in the BIG file.

   Revision 1.16  1997/09/01 02:39:52  Jeff_Weston
   - The palette is now loaded from PALETTE.BMP from the BIG data file.

   Revision 1.15  1997/08/23 01:36:49  Jeff_Weston
   - Fixed bug with certain map tiles not writing over certain tiles.

   Revision 1.14  1997/08/23 01:02:59  Jeff_Weston
   - Code now fully supports any number of land tiles.
   - Added code to scroll through the land tiles.

   Revision 1.13  1997/08/22 23:44:31  Jeff_Weston
   - Added code to allow the number of land tiles to be variable.

   Revision 1.12  1997/08/22 05:56:24  Jeff_Weston
   - The editor now loads all bitmaps from the external BIG file.
   - Moved all constant strings into the header file.

   Revision 1.11  1997/08/22 04:03:46  Jeff_Weston
   - Added code to load land bitmaps from external BIG file.

   Revision 1.10  1997/07/30 00:39:44  Jeff_Weston
   - Added save and load functionality.
   - Changed phase display to be optional.

   Revision 1.9  1997/07/24 07:41:13  Jeff_Weston
   - Added code to support Digit Bitmaps.
   - Added Location Display code.
   - Added Phase Display code.

   Revision 1.8  1997/07/23 22:31:07  Jeff_Weston
   - Added code support for 'Click and Drag' map editing.

   Revision 1.7  1997/07/23 06:58:52  Jeff_Weston
   - Added Map Scrolling code.
   - Initialization Code now sets up the editor with the standard palette.

   Revision 1.6  1997/07/23 01:08:32  Jeff_Weston
   - Implemented map editor code with no scrolling.

   Revision 1.5  1997/07/22 23:26:45  Jeff_Weston
   - Added functionality to select Land Bitmaps using the mouse.

   Revision 1.4  1997/07/22 21:59:25  Jeff_Weston
   - Moved DirectX error messaging outside of editor.cpp.
   - Added Land Select supporting code.

   Revision 1.3  1997/07/22 07:40:20  Jeff_Weston
   - Removed Front and Back buffer messages.
   - Cycles through every Land Bitmap on display.

   Revision 1.2  1997/07/22 06:08:16  Jeff_Weston
   - Added code to Load and CleanUp Initial Version of Land Bitmaps.

   Revision 1.1  1997/07/22 00:06:30  Jeff_Weston
   Initial revision


*********************************************************************/

#include "editor.h"

APPLICATION Application;

APPLICATION::APPLICATION () : lpDDSLand (0)
{
}

LRESULT CALLBACK APPLICATION::WindProc (HWND hWnd, UINT iMsg,
                                        WPARAM wParam, LPARAM lParam)
{
  switch (iMsg)
  {
    case WM_ACTIVATEAPP : bActive = wParam; break;

    case WM_TIMER : Timer (); break;

    case WM_KEYDOWN : KeyDown (wParam); break;

    case WM_LBUTTONDOWN :
      LButtonDown (LOWORD (lParam), HIWORD (lParam), 0); return 0;

    case WM_LBUTTONUP : bLMButtonDown = FALSE; return 0;

    case WM_MOUSEMOVE : MouseMove (lParam); return 0;

    case WM_DESTROY : PostQuitMessage (0); CleanUp (); return 0;
  }
  return DefWindowProc (hWnd, iMsg, wParam, lParam);
}

int APPLICATION::Run (HINSTANCE hInstance, int iCmdShow)
{
  MSG        msg;

  if (!Init (hInstance, iCmdShow)) return FALSE;

  while (GetMessage (&msg, NULL, 0, 0))
  {
    TranslateMessage (&msg);
    DispatchMessage (&msg);
  }
  return msg.wParam;
}

BOOL APPLICATION::Init (HINSTANCE hInstance, int iCmdShow)
{
  DDSURFACEDESC       ddsd;
  DDSCAPS             ddscaps;
  HRESULT             ddrval;
  DDCOLORKEY          DDColorKey;
  int                 iLoop, iError, iX, iY, i;
  char                buf[BUFFER];

  static char szFilter[] = "Dominate Map Files (*.map)\0*.map\0" \
                           "All Files (*.*)\0*.*\0\0";

  bLMButtonDown = FALSE;

  iPhase = 0;

  iCurRow = 0;

  ptSelect.x = 0;
  ptSelect.y = 0;
  iSelectedTile = 0;

  iMapCornerY = (MAP_SIZE_X / 2) - (MAP_VIEW_X / 2);
  iMapCornerX = (MAP_SIZE_Y / 2) - (MAP_VIEW_Y / 2);

  iMouseX = MODE_X / 2;
  iMouseY = MODE_Y / 2;

  for (iY = 0; iY < SELECT_WIN_Y; iY++)
    for (iX = 0; iX < SELECT_WIN_X; iX++)
    {
      /* Setup Selection Rectangles *********************************/
      rcSelectRect[iX][iY].left   = ((iX * (LAND_X + 2)) + 1) + SELECT_DISP_X;
      rcSelectRect[iX][iY].top    = ((iY * (LAND_Y + 2)) + 1) + SELECT_DISP_Y;
      rcSelectRect[iX][iY].right  = rcSelectRect[iX][iY].left + LAND_X;
      rcSelectRect[iX][iY].bottom = rcSelectRect[iX][iY].top  + LAND_Y;

      /* Setup Bitmap Selection Click Events ************************/
      i = (iX * SELECT_WIN_Y) + iY;
      ceClickEvents[i].rcClickRect = rcSelectRect[iX][iY];
      ceClickEvents[i].iFunction = 1;
      ceClickEvents[i].iX = iX;
      ceClickEvents[i].iY = iY;
    }

  for (iY = 0; iY < MAP_VIEW_Y; iY++)
    for (iX = 0; iX < MAP_VIEW_X; iX++)
    {
      /* Setup Map View Rectangles **********************************/
      rcMapViewRect[iX][iY].top    = MAP_VIEW_START_Y + (iY * LAND_Y);
      rcMapViewRect[iX][iY].left   = MAP_VIEW_START_X + (iX * LAND_X);
      rcMapViewRect[iX][iY].bottom = rcMapViewRect[iX][iY].top  + LAND_Y;
      rcMapViewRect[iX][iY].right  = rcMapViewRect[iX][iY].left + LAND_X;

      /* Setup Map View Click Events ********************************/
      i = ((iX * MAP_VIEW_Y) + iY) + 12;
      ceClickEvents[i].rcClickRect = rcMapViewRect[iX][iY];
      ceClickEvents[i].iFunction = 2;
      ceClickEvents[i].iX = iX;
      ceClickEvents[i].iY = iY;
    }

  for (iY = 0; iY < MAP_SIZE_Y; iY++)
    for (iX = 0; iX < MAP_SIZE_X; iX++)
      iMapTile[iX][iY] = -1;

  ceClickEvents[LOAD_EVENT].rcClickRect.top    = LOAD_DISP_Y;
  ceClickEvents[LOAD_EVENT].rcClickRect.left   = LOAD_DISP_X;
  ceClickEvents[LOAD_EVENT].rcClickRect.bottom = LOAD_DISP_Y + LOAD_BUTTON_Y;
  ceClickEvents[LOAD_EVENT].rcClickRect.right  = LOAD_DISP_X + LOAD_BUTTON_X;
  ceClickEvents[LOAD_EVENT].iFunction = 3;
  ceClickEvents[LOAD_EVENT].iX = 1;
  ceClickEvents[LOAD_EVENT].iY = 0;

  ceClickEvents[SAVE_EVENT].rcClickRect.top    = SAVE_DISP_Y;
  ceClickEvents[SAVE_EVENT].rcClickRect.left   = SAVE_DISP_X;
  ceClickEvents[SAVE_EVENT].rcClickRect.bottom = SAVE_DISP_Y + SAVE_BUTTON_Y;
  ceClickEvents[SAVE_EVENT].rcClickRect.right  = SAVE_DISP_X + SAVE_BUTTON_X;
  ceClickEvents[SAVE_EVENT].iFunction = 3;
  ceClickEvents[SAVE_EVENT].iX = 2;
  ceClickEvents[SAVE_EVENT].iY = 0;

  ceClickEvents[DOWN_EVENT].rcClickRect.top    = DOWN_DISP_Y;
  ceClickEvents[DOWN_EVENT].rcClickRect.left   = DOWN_DISP_X;
  ceClickEvents[DOWN_EVENT].rcClickRect.bottom = DOWN_DISP_Y + DOWN_BUTTON_Y;
  ceClickEvents[DOWN_EVENT].rcClickRect.right  = DOWN_DISP_X + DOWN_BUTTON_X;
  ceClickEvents[DOWN_EVENT].iFunction = 3;
  ceClickEvents[DOWN_EVENT].iX = 3;
  ceClickEvents[DOWN_EVENT].iY = 0;

  ceClickEvents[UP_EVENT].rcClickRect.top    = UP_DISP_Y;
  ceClickEvents[UP_EVENT].rcClickRect.left   = UP_DISP_X;
  ceClickEvents[UP_EVENT].rcClickRect.bottom = UP_DISP_Y + UP_BUTTON_Y;
  ceClickEvents[UP_EVENT].rcClickRect.right  = UP_DISP_X + UP_BUTTON_X;
  ceClickEvents[UP_EVENT].iFunction = 3;
  ceClickEvents[UP_EVENT].iX = 4;
  ceClickEvents[UP_EVENT].iY = 0;

  ofn.lStructSize       = sizeof (OPENFILENAME);
  ofn.hwndOwner         = hWnd;
  ofn.hInstance         = NULL;
  ofn.lpstrFilter       = szFilter;
  ofn.lpstrCustomFilter = NULL;
  ofn.nMaxCustFilter    = 0;
  ofn.nFilterIndex      = 0;
  ofn.lpstrFile         = NULL;
  ofn.nMaxFile          = PATH_BUFFER;
  ofn.lpstrFileTitle    = NULL;
  ofn.nMaxFileTitle     = BUFFER;
  ofn.Flags             = 0;
  ofn.nFileExtension    = 0;
  ofn.lpstrDefExt       = MAP_EXT;
  ofn.lCustData         = 0L;
  ofn.lpfnHook          = NULL;
  ofn.lpTemplateName    = NULL;

  WndClass.cbSize        = sizeof (WndClass);
  WndClass.style         = CS_HREDRAW | CS_VREDRAW;
  WndClass.lpfnWndProc   = WndProc;
  WndClass.cbClsExtra    = 0;
  WndClass.cbWndExtra    = 0;
  WndClass.hInstance     = hInstance;
  WndClass.hIcon         = LoadIcon (NULL, IDI_APPLICATION);
//  WndClass.hCursor       = LoadCursor (NULL, IDC_ARROW);
  WndClass.hCursor       = NULL;
  WndClass.hbrBackground = GetStockObject (BLACK_BRUSH);
  WndClass.lpszMenuName  = NULL;
  WndClass.lpszClassName = NAME;
  WndClass.hIconSm       = LoadIcon (NULL, IDI_APPLICATION);

  RegisterClassEx (&WndClass);


  hWnd = CreateWindowEx (WS_EX_TOPMOST,
                         NAME,
                         TITLE,
                         WS_POPUP,
                         0,
                         0,
                         GetSystemMetrics (SM_CXSCREEN),
                         GetSystemMetrics (SM_CYSCREEN),
                         NULL,
                         NULL,
                         hInstance,
                         NULL);

/*
  hWnd = CreateWindowEx (NULL,
                         NAME,
                         TITLE,
                         WS_OVERLAPPEDWINDOW,
                         0,
                         0,
                         MODE_X,
                         MODE_Y,
                         NULL,
                         NULL,
                         hInstance,
                         NULL);
*/
  if (!hWnd) return FALSE;

  ShowWindow (hWnd, iCmdShow);
  UpdateWindow (hWnd);

  hOldCursor = SetCursor (NULL); 

  ddrval = DirectDrawCreate (NULL, &lpDD, NULL);
  if (ddrval == DD_OK)
  {
    ddrval = lpDD->SetCooperativeLevel (hWnd, DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN);
//    ddrval = lpDD->SetCooperativeLevel (hWnd, DDSCL_NORMAL);
    if (ddrval == DD_OK)
    {
      ddrval = lpDD->SetDisplayMode (MODE_X, MODE_Y, MODE_B);
      if (ddrval == DD_OK)
      {
        ddsd.dwSize = sizeof (ddsd);

        ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
//        ddsd.dwFlags = DDSD_CAPS;

        ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE | DDSCAPS_FLIP | DDSCAPS_COMPLEX;
//        ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

        ddsd.dwBackBufferCount = 1;
        ddrval = lpDD->CreateSurface (&ddsd, &lpDDSPrimary, NULL);
        if (ddrval == DD_OK)
        {
          ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
//          ddsd.dwFlags = DDSD_HEIGHT | DDSD_WIDTH | DDSD_CAPS;
//          ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
//          ddsd.dwHeight = MODE_Y;
//          ddsd.dwWidth  = MODE_X;

          ddrval = lpDDSPrimary->GetAttachedSurface (&ddscaps, &lpDDSBack);
//          ddrval = lpDD->CreateSurface (&ddsd, &lpDDSBack, NULL);
          if (ddrval == DD_OK)
          {
            iError = 0;

            ddrval = lpDDSPrimary->GetPalette (&lpDDPalSystem);
            lpDDPalGame = BIG_DDLoadPalette (BIG_FILE, lpDD, "PALETTE.BMP");
//            lpDDPalGame = DDLoadPalette (lpDD, NULL);
            if (lpDDPalGame == NULL)
            {
              iError = 1;
              PrintError (ddrval, hWnd);
            }
            else
            {
              ddrval = lpDDSPrimary->SetPalette (lpDDPalGame);
              if (ddrval != DD_OK) iError = 1;
            }

            iLandBitmaps = 0;
            wsprintf (buf, LAND_BMP, iLandBitmaps + 1);
            while (BIG_FileExists (BIG_FILE, buf))
            {
              iLandBitmaps++;
              wsprintf (buf, LAND_BMP, iLandBitmaps + 1);
            }

            if (iLandBitmaps % 2 == 1) iLandBitmaps++;

            lpDDSLand.setSize (iLandBitmaps);

            for (iLoop = 0; iLoop < iLandBitmaps; iLoop++)
            {
              wsprintf (buf, LAND_BMP, iLoop + 1);
              if (BIG_FileExists (BIG_FILE, buf))
              {
                lpDDSLand[iLoop] = BIG_DDLoadBitmap (BIG_FILE, lpDD,
                                                     buf, LAND_X, LAND_Y);
                if (lpDDSLand[iLoop] == NULL) iError = 1;
              }
              else
              {
                lpDDSLand[iLoop] = BIG_DDLoadBitmap (BIG_FILE, lpDD,
                                                     UNUSED, LAND_X, LAND_Y);
                if (lpDDSLand[iLoop] == NULL) iError = 1;
              }
            }

            for (iLoop = 0; iLoop < DIGIT_BITMAPS; iLoop++)
            {
              wsprintf (buf, DIGIT_BMP, iLoop);
              lpDDSDigit[iLoop] = BIG_DDLoadBitmap (BIG_FILE, lpDD, buf,
                                                    DIGIT_X, DIGIT_Y);
              if (lpDDSDigit[iLoop] == NULL) iError = 1;
            }

            lpDDSSelect = BIG_DDLoadBitmap (BIG_FILE, lpDD, SELECT_BMP,
                                            SELECT_X, SELECT_Y);
            if (lpDDSSelect == NULL) iError = 1;

            lpDDSLoadButton = BIG_DDLoadBitmap (BIG_FILE, lpDD, BUT_LOAD_BMP,
                                                LOAD_BUTTON_X, LOAD_BUTTON_Y);
            if (lpDDSLoadButton == NULL) iError = 1;

            lpDDSSaveButton = BIG_DDLoadBitmap (BIG_FILE, lpDD, BUT_SAVE_BMP,
                                                SAVE_BUTTON_X, SAVE_BUTTON_Y);
            if (lpDDSSaveButton == NULL) iError = 1;

            lpDDSDownButton = BIG_DDLoadBitmap (BIG_FILE, lpDD, BUT_DOWN_BMP,
                                                DOWN_BUTTON_X, DOWN_BUTTON_Y);
            if (lpDDSDownButton == NULL) iError = 1;

            lpDDSUpButton = BIG_DDLoadBitmap (BIG_FILE, lpDD, BUT_UP_BMP,
                                                UP_BUTTON_X, UP_BUTTON_Y);
            if (lpDDSUpButton == NULL) iError = 1;

            lpDDSCursor = BIG_DDLoadBitmap (BIG_FILE, lpDD, CURSOR_BMP,
                                            CURSOR_X, CURSOR_Y);
            if (lpDDSUpButton == NULL) iError = 1;

            else
            {
              DDColorKey.dwColorSpaceLowValue = 0;
              DDColorKey.dwColorSpaceHighValue = 0;
              ddrval = lpDDSSelect->SetColorKey (DDCKEY_SRCBLT, &DDColorKey);
              if (ddrval != DD_OK)
              {
                iError = 1;
                PrintError (ddrval, hWnd);
              }
              ddrval = lpDDSCursor->SetColorKey (DDCKEY_SRCBLT, &DDColorKey);
              if (ddrval != DD_OK)
              {
                iError = 1;
                PrintError (ddrval, hWnd);
              }
            }

            if (SetTimer (hWnd, TIMER_ID, TIMER_RATE, NULL) == NULL)
              iError = 1;

            if (iError) return FALSE;
            else
            {
              UpdateScreen ();
              return TRUE;
            }
          }
        }
      }
    }
  }
  PrintError (ddrval, hWnd);
  CleanUp ();
  DestroyWindow (hWnd);
  return FALSE;
}

void APPLICATION::CleanUp (void)
{
  int iLoop;

  SetCursor (hOldCursor);

  if (lpDD != NULL)
  {
    if (lpDDSPrimary != NULL)
    {
      lpDDSPrimary->Release ();
      lpDDSPrimary = NULL;
    }
    for (iLoop = 0; iLoop > iLandBitmaps; iLoop++)
    {
      if (lpDDSLand[iLoop] != NULL)
      {
        lpDDSLand[iLoop]->Release ();
        lpDDSLand[iLoop] = NULL;
      }
    }
    for (iLoop = 0; iLoop > DIGIT_BITMAPS; iLoop++)
    {
      if (lpDDSDigit[iLoop] != NULL)
      {
        lpDDSDigit[iLoop]->Release ();
        lpDDSDigit[iLoop] = NULL;
      }
    }

    if (lpDDSSelect != NULL)
    {
      lpDDSSelect->Release ();
      lpDDSSelect = NULL;
    }

    lpDD->Release ();
    lpDD = NULL;
  }
}

void APPLICATION::SelectionDisplay ()
{
  RECT     rcSrcRect, rcDestRect;
  int     iX, iY, iCurTile;

  iCurTile = iCurRow * SELECT_WIN_X;

  rcSrcRect.left   = 0;
  rcSrcRect.top    = 0;
  rcSrcRect.right  = LAND_X;
  rcSrcRect.bottom = LAND_Y;

  for (iY = 0; iY < SELECT_WIN_Y; iY++)
    for (iX = 0; iX < SELECT_WIN_X; iX++)
    {
    lpDDSBack->Blt (&rcSelectRect[iX][iY],
                    lpDDSLand[iCurTile],
                    &rcSrcRect, DDBLT_WAIT, NULL);
    if (++iCurTile == iLandBitmaps) iCurTile = 0;
    }
  rcSrcRect.left   = 0;
  rcSrcRect.top    = 0;
  rcSrcRect.right  = SELECT_X;
  rcSrcRect.bottom = SELECT_Y;

  rcDestRect.left   = rcSelectRect[ptSelect.x][ptSelect.y].left   - 1;
  rcDestRect.top    = rcSelectRect[ptSelect.x][ptSelect.y].top    - 1;
  rcDestRect.right  = rcSelectRect[ptSelect.x][ptSelect.y].right  + 1;
  rcDestRect.bottom = rcSelectRect[ptSelect.x][ptSelect.y].bottom + 1;

  lpDDSBack->Blt (&rcDestRect, lpDDSSelect, &rcSrcRect,
                  DDBLT_WAIT |  DDBLT_KEYSRC, NULL);
}

void APPLICATION::MapViewDisplay ()
{
  RECT     rcSrcRect;
  int     iX, iY;

  rcSrcRect.left   = 0;
  rcSrcRect.top    = 0;
  rcSrcRect.right  = LAND_X;
  rcSrcRect.bottom = LAND_Y;

  for (iX = 0; iX < MAP_VIEW_X; iX++)
    for (iY = 0; iY < MAP_VIEW_Y; iY++)
    {
      if (iMapTile[iMapCornerX + iX][iMapCornerY + iY] > -1)
        lpDDSBack->Blt (&rcMapViewRect[iX][iY],
                        lpDDSLand[iMapTile[iMapCornerX + iX][iMapCornerY + iY]],
                        &rcSrcRect, DDBLT_WAIT, NULL);
    }
}

void APPLICATION::DigitDisplay ()
{
  RECT rcSrcRect, rcDestRect;
  int i;

  rcSrcRect.top    = 0;
  rcSrcRect.left   = 0;
  rcSrcRect.bottom = DIGIT_Y;
  rcSrcRect.right  = DIGIT_X;

  rcDestRect.top    = DIGIT_DISP_Y;
  rcDestRect.left   = DIGIT_DISP_X;
  rcDestRect.bottom = rcDestRect.top  + DIGIT_Y;
  rcDestRect.right  = rcDestRect.left + DIGIT_X;
  i = iCurX / 100;
  lpDDSBack->Blt (&rcDestRect, lpDDSDigit[i], &rcSrcRect, DDBLT_WAIT, NULL);

  rcDestRect.left  += DIGIT_X;
  rcDestRect.right += DIGIT_X;
  i = (iCurX % 100) / 10;
  lpDDSBack->Blt (&rcDestRect, lpDDSDigit[i], &rcSrcRect, DDBLT_WAIT, NULL);

  rcDestRect.left  += DIGIT_X;
  rcDestRect.right += DIGIT_X;
  i = iCurX % 10;
  lpDDSBack->Blt (&rcDestRect, lpDDSDigit[i], &rcSrcRect, DDBLT_WAIT, NULL);

  rcDestRect.left  += DIGIT_X;
  rcDestRect.right += DIGIT_X;

  rcDestRect.left  += DIGIT_X;
  rcDestRect.right += DIGIT_X;
  i = iCurY / 100;
  lpDDSBack->Blt (&rcDestRect, lpDDSDigit[i], &rcSrcRect, DDBLT_WAIT, NULL);

  rcDestRect.left  += DIGIT_X;
  rcDestRect.right += DIGIT_X;
  i = (iCurY % 100) / 10;
  lpDDSBack->Blt (&rcDestRect, lpDDSDigit[i], &rcSrcRect, DDBLT_WAIT, NULL);

  rcDestRect.left  += DIGIT_X;
  rcDestRect.right += DIGIT_X;
  i = iCurY % 10;
  lpDDSBack->Blt (&rcDestRect, lpDDSDigit[i], &rcSrcRect, DDBLT_WAIT, NULL);
}

void APPLICATION::PhaseDisplay ()
{
  RECT rcSrcRect, rcDestRect;

  rcSrcRect.top    = 0;
  rcSrcRect.left   = 0;
  rcSrcRect.bottom = DIGIT_Y;
  rcSrcRect.right  = DIGIT_X;

  rcDestRect.top    = PHASE_DISP_Y;
  rcDestRect.left   = PHASE_DISP_X;
  rcDestRect.bottom = rcDestRect.top  + DIGIT_Y;
  rcDestRect.right  = rcDestRect.left + DIGIT_X;

  lpDDSBack->Blt (&rcDestRect, lpDDSDigit[iPhase],
                  &rcSrcRect, DDBLT_WAIT, NULL);
}

void APPLICATION::ButtonDisplay ()
{
  RECT     rcSrcRect, rcDestRect;

  rcSrcRect.left   = 0;
  rcSrcRect.top    = 0;
  rcSrcRect.right  = LOAD_BUTTON_X;
  rcSrcRect.bottom = LOAD_BUTTON_Y;

  rcDestRect.left   = LOAD_DISP_X;
  rcDestRect.top    = LOAD_DISP_Y;
  rcDestRect.right  = LOAD_DISP_X + LOAD_BUTTON_X;
  rcDestRect.bottom = LOAD_DISP_Y + LOAD_BUTTON_Y;

  lpDDSBack->Blt (&rcDestRect, lpDDSLoadButton,
                  &rcSrcRect, DDBLT_WAIT, NULL);

  rcSrcRect.left   = 0;
  rcSrcRect.top    = 0;
  rcSrcRect.right  = SAVE_BUTTON_X;
  rcSrcRect.bottom = SAVE_BUTTON_Y;

  rcDestRect.left   = SAVE_DISP_X;
  rcDestRect.top    = SAVE_DISP_Y;
  rcDestRect.right  = SAVE_DISP_X + SAVE_BUTTON_X;
  rcDestRect.bottom = SAVE_DISP_Y + SAVE_BUTTON_Y;

  lpDDSBack->Blt (&rcDestRect, lpDDSSaveButton,
                  &rcSrcRect, DDBLT_WAIT, NULL);

  rcSrcRect.left   = 0;
  rcSrcRect.top    = 0;
  rcSrcRect.right  = DOWN_BUTTON_X;
  rcSrcRect.bottom = DOWN_BUTTON_Y;

  rcDestRect.left   = DOWN_DISP_X;
  rcDestRect.top    = DOWN_DISP_Y;
  rcDestRect.right  = DOWN_DISP_X + DOWN_BUTTON_X;
  rcDestRect.bottom = DOWN_DISP_Y + DOWN_BUTTON_Y;

  lpDDSBack->Blt (&rcDestRect, lpDDSDownButton,
                  &rcSrcRect, DDBLT_WAIT, NULL);

  rcSrcRect.left   = 0;
  rcSrcRect.top    = 0;
  rcSrcRect.right  = UP_BUTTON_X;
  rcSrcRect.bottom = UP_BUTTON_Y;

  rcDestRect.left   = UP_DISP_X;
  rcDestRect.top    = UP_DISP_Y;
  rcDestRect.right  = UP_DISP_X + UP_BUTTON_X;
  rcDestRect.bottom = UP_DISP_Y + UP_BUTTON_Y;

  lpDDSBack->Blt (&rcDestRect, lpDDSUpButton,
                  &rcSrcRect, DDBLT_WAIT, NULL);
}

void APPLICATION::CursorDisplay ()
{
  RECT rcSrcRect, rcDestRect;

  rcSrcRect.left   = 0;
  rcSrcRect.top    = 0;
  rcSrcRect.right  = CURSOR_X;
  rcSrcRect.bottom = CURSOR_Y;

  rcDestRect.left   = iMouseX;
  rcDestRect.top    = iMouseY;
  rcDestRect.right  = iMouseX + CURSOR_X;
  rcDestRect.bottom = iMouseY + CURSOR_Y;

  lpDDSBack->Blt (&rcDestRect, lpDDSCursor,
                  &rcSrcRect, DDBLT_WAIT | DDBLT_KEYSRC, NULL);
}

void APPLICATION::UpdateScreen ()
{
  DDBLTFX ddbltfx;
  HRESULT ddrval;

  SetCursor (NULL);

  if (iPhase) iPhase = 0;
  else iPhase = 1;

  ddbltfx.dwSize = sizeof(ddbltfx);
  ddbltfx.dwFillColor = 0;

  ddrval = lpDDSBack->Blt (NULL, NULL, NULL, DDBLT_COLORFILL | DDBLT_WAIT,
                           &ddbltfx);
  if (ddrval != DD_OK) PrintError (ddrval, hWnd);

  SelectionDisplay ();
  MapViewDisplay ();
  DigitDisplay ();
  ButtonDisplay ();

#pragma warn -ccc
#pragma warn -rch
  if (PHASE_ON) PhaseDisplay ();
#pragma warn +rch
#pragma warn +ccc

  CursorDisplay ();

  ddrval = lpDDSPrimary->Flip (NULL, DDFLIP_WAIT);
  if (ddrval != DD_OK)
  {
    if (ddrval == DDERR_SURFACELOST)
    {
      lpDDSPrimary->Restore ();
      PrintError (ddrval, hWnd);
    }
  }
}

void APPLICATION::SaveMap ()
{
  HANDLE        hSaveFile;
  unsigned long iBytesWritten, iTotal;
  unsigned char iMapVersion, iMapX, iMapY;
  int           iX, iY;
  char          szFileName[PATH_BUFFER];
  char          szFileTitle[BUFFER];

  if (iPhase) UpdateScreen ();

  lpDDSPrimary->SetPalette (lpDDPalSystem);

  InvalidateRect (hWnd, NULL, TRUE);

  ofn.hwndOwner      = hWnd;
  ofn.lpstrFile      = szFileName;
  ofn.lpstrFileTitle = szFileTitle;
  ofn.Flags          = OFN_OVERWRITEPROMPT |
                       OFN_HIDEREADONLY    |
                       OFN_NOCHANGEDIR;

  wsprintf (szFileName, NULL_STR);
  wsprintf (szFileTitle, NULL_STR);

  if (GetSaveFileName (&ofn) == TRUE)
  {
    iMapVersion = MAP_FILE_VERSION;
    iMapX = MAP_SIZE_X;
    iMapY = MAP_SIZE_Y;
    iTotal = 0;

    hSaveFile = CreateFile(ofn.lpstrFile, GENERIC_WRITE, 0, NULL,
                 CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, NULL);

    if (hSaveFile == INVALID_HANDLE_VALUE)
      MessageBox (hWnd, SAVE_ERROR_1, SAVE_ERROR, MB_OK);
    else
    {
      WriteFile (hSaveFile, &iMapVersion, sizeof (char),
                 &iBytesWritten, NULL);
      iTotal += iBytesWritten;

      WriteFile (hSaveFile, &iMapX, sizeof (char),
                 &iBytesWritten, NULL);
      iTotal += iBytesWritten;

      WriteFile (hSaveFile, &iMapY, sizeof (char),
                 &iBytesWritten, NULL);
      iTotal += iBytesWritten;

      for (iY = 0; iY < MAP_SIZE_Y; iY++)
        for (iX = 0; iX < MAP_SIZE_X; iX++)
        {
          WriteFile (hSaveFile, &iMapTile[iX][iY], sizeof (char),
                     &iBytesWritten, NULL);
          iTotal += iBytesWritten;
        }

      if (iTotal != (sizeof (char) * 3) +
                    (MAP_SIZE_X * MAP_SIZE_Y * sizeof (char)))
        MessageBox (hWnd, SAVE_ERROR_2, SAVE_ERROR, MB_OK);
      else
        MessageBox (hWnd, SAVE_OK_1, SAVE_OK, MB_OK);

      CloseHandle (hSaveFile);
    }
  }

  lpDDSPrimary->SetPalette (lpDDPalGame);

  UpdateScreen ();
}

void APPLICATION::LoadMap ()
{
  HANDLE        hLoadFile;
  unsigned long iBytesRead, iTotal;
  unsigned char iMapVersion, iMapX, iMapY;
  int           iX, iY;
  char          szFileName[PATH_BUFFER];
  char          szFileTitle[BUFFER];
  char          iTempMapTile[MAP_SIZE_X][MAP_SIZE_Y];

  if (iPhase) UpdateScreen ();

  lpDDSPrimary->SetPalette (lpDDPalSystem);

  InvalidateRect (hWnd, NULL, TRUE);

  ofn.hwndOwner      = hWnd;
  ofn.lpstrFile      = szFileName;
  ofn.lpstrFileTitle = szFileTitle;
  ofn.Flags          = OFN_OVERWRITEPROMPT |
                       OFN_HIDEREADONLY    |
                       OFN_NOCHANGEDIR;

  wsprintf (szFileName, NULL_STR);
  wsprintf (szFileTitle, NULL_STR);

  if (GetOpenFileName (&ofn) == TRUE)
  {
    iTotal = 0;

    hLoadFile = CreateFile(ofn.lpstrFile, GENERIC_READ, 0, NULL,
                 OPEN_EXISTING, FILE_ATTRIBUTE_ARCHIVE, NULL);

    if (hLoadFile == INVALID_HANDLE_VALUE)
      MessageBox (hWnd, LOAD_ERROR_1, LOAD_ERROR, MB_OK);
    else
    {
      ReadFile (hLoadFile, &iMapVersion, sizeof (char),
                &iBytesRead, NULL);
      iTotal += iBytesRead;

      ReadFile (hLoadFile, &iMapX, sizeof (char),
                 &iBytesRead, NULL);
      iTotal += iBytesRead;

      ReadFile (hLoadFile, &iMapY, sizeof (char),
                 &iBytesRead, NULL);
      iTotal += iBytesRead;

      if (iTotal != sizeof (char) * 3)
        MessageBox (hWnd, LOAD_ERROR_2, LOAD_ERROR, MB_OK);
      else
      {
        if (iMapVersion != MAP_FILE_VERSION)
          MessageBox (hWnd, LOAD_ERROR_3, LOAD_ERROR, MB_OK);
        else
        {
          if ((iMapX != MAP_SIZE_X) ||
              (iMapY != MAP_SIZE_Y))
            MessageBox (hWnd, LOAD_ERROR_4, LOAD_ERROR, MB_OK);
          else
          {
            iTotal = 0;
            for (iY = 0; iY < MAP_SIZE_Y; iY++)
              for (iX = 0; iX < MAP_SIZE_X; iX++)
              {
                ReadFile (hLoadFile, &iTempMapTile[iX][iY], sizeof (char),
                          &iBytesRead, NULL);
                iTotal += iBytesRead;
              }
            if (iTotal != sizeof (char) * MAP_SIZE_X * MAP_SIZE_Y)
              MessageBox (hWnd, LOAD_ERROR_5, LOAD_ERROR, MB_OK);
            else
            {
              for (iY = 0; iY < MAP_SIZE_Y; iY++)
                for (iX = 0; iX < MAP_SIZE_X; iX++)
                  iMapTile[iX][iY] = iTempMapTile[iX][iY];
              MessageBox (hWnd, LOAD_OK_1, LOAD_OK, MB_OK);
            }
          }
        }
      }
      CloseHandle (hLoadFile);
    }
  }

  lpDDSPrimary->SetPalette (lpDDPalGame);

  UpdateScreen ();
}

void APPLICATION::ScrollD ()
{
  if (++iCurRow * SELECT_WIN_X >= iLandBitmaps) iCurRow = 0;
  iSelectedTile = ((ptSelect.y + iCurRow) * SELECT_WIN_X) + ptSelect.x;
  while (iSelectedTile >= iLandBitmaps) iSelectedTile -= iLandBitmaps;
  UpdateScreen ();
}

void APPLICATION::ScrollU ()
{
  if (--iCurRow < 0) iCurRow = (iLandBitmaps - 1) / 2;
  iSelectedTile = ((ptSelect.y + iCurRow) * SELECT_WIN_X) + ptSelect.x;
  while (iSelectedTile >= iLandBitmaps) iSelectedTile -= iLandBitmaps;
  UpdateScreen ();
}

void APPLICATION::Click1 (int iX, int iY)
{
  if (bLMButtonDown == FALSE)
  {
    iSelectedTile = ((iY + iCurRow) * SELECT_WIN_X) + iX;
    while (iSelectedTile >= iLandBitmaps) iSelectedTile -= iLandBitmaps;
    ptSelect.x = iX;
    ptSelect.y = iY;
    UpdateScreen ();
  }
}

void APPLICATION::Click2 (int iX, int iY)
{
  int iChange;

  iChange = 0;

  iLastX = iX;
  iLastY = iY;

  if (bLMButtonDown)
    if (iMapTile[iMapCornerX + iX][iMapCornerY + iY] != iSelectedTile)
    {

#pragma warn -sig
      iMapTile[iMapCornerX + iX][iMapCornerY + iY] = iSelectedTile;
#pragma warn +sig

      iChange = 1;
    }

  if ((iCurX != iMapCornerX + iX) ||
      (iCurY != iMapCornerY + iY))
  {
    iChange = 1;
    iCurX = iMapCornerX + iX;
    iCurY = iMapCornerY + iY;
  }
  if (iChange) UpdateScreen ();
}

void APPLICATION::Click3 (int iX, int iY)
{
  if (iY == 0)
  {
    switch (iX)
    {
      case 1 : LoadMap (); break;
      case 2 : SaveMap (); break;
      case 3 : ScrollD (); break;
      case 4 : ScrollU (); break;
    }
  }
}

void APPLICATION::Timer ()
{
  if (bActive)
  {
    if (iMouseX <          5) iMapCornerX--;
    if (iMouseX > MODE_X - 5) iMapCornerX++;
    if (iMouseY <          5) iMapCornerY--;
    if (iMouseY > MODE_Y - 5) iMapCornerY++;

    if (iMapCornerX > MAP_SIZE_X - MAP_VIEW_X)
      iMapCornerX = MAP_SIZE_X - MAP_VIEW_X;

    if (iMapCornerY > MAP_SIZE_Y - MAP_VIEW_Y)
      iMapCornerY = MAP_SIZE_Y - MAP_VIEW_Y;

    if (iMapCornerX < 0) iMapCornerX = 0;
    if (iMapCornerY < 0) iMapCornerY = 0;

    Click2 (iLastX, iLastY);
  }
}

void APPLICATION::KeyDown (WPARAM wParam)
{
  switch (wParam)
  {
    case VK_ESCAPE :
    case VK_F12 :
    case VK_SPACE :
      PostMessage (hWnd, WM_CLOSE, 0, 0);
      break;

    case 0x53 :
      SaveMap ();
      break;

    case 0x4C :
      LoadMap ();
      break;
  }
}

void APPLICATION::LButtonDown (int iX, int iY, int iMouseMove)
{
  int iLoop;

  for (iLoop = 0; iLoop < CLICK_EVENTS; iLoop++)
  {
    if ((ceClickEvents[iLoop].rcClickRect.top    < iY) &&
        (ceClickEvents[iLoop].rcClickRect.left   < iX) &&
        (ceClickEvents[iLoop].rcClickRect.bottom > iY) &&
        (ceClickEvents[iLoop].rcClickRect.right  > iX))
    {
      switch (ceClickEvents[iLoop].iFunction)
      {
        case 1 : if (iMouseMove == 0)
                   Click1 (ceClickEvents[iLoop].iX, ceClickEvents[iLoop].iY);
                 break;
        case 2 : if (iMouseMove == 0) bLMButtonDown = TRUE;
                 Click2 (ceClickEvents[iLoop].iX, ceClickEvents[iLoop].iY);
                 break;
        case 3 : if (iMouseMove == 0)
                   Click3 (ceClickEvents[iLoop].iX, ceClickEvents[iLoop].iY);
                 break;
      }
    }
  }
}

void APPLICATION::MouseMove (LPARAM lParam)
{
  iMouseX = LOWORD (lParam);
  iMouseY = HIWORD (lParam);

  UpdateScreen ();

  if (bLMButtonDown) LButtonDown (iMouseX, iMouseY, 0);
  else               LButtonDown (iMouseX, iMouseY, 1);
}

LRESULT CALLBACK WndProc (HWND hWnd, UINT iMsg,
                           WPARAM wParam, LPARAM lParam)
{
  return Application.WindProc (hWnd, iMsg, wParam, lParam);
}

#pragma argsused

int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
{
  return Application.Run (hInstance, iCmdShow);
}
