/*********************************************************************

   Group   - Apocalypse Games

   Project - Dominate

   Target  - editor.exe
   File    - editor.h
   Author  - Jeff Weston

   Purpose - Header File for Map Editor for Dominate

**********************************************************************

RCS Information :

   $Id: editor.h 1.15 1997/09/02 07:36:36 Jeff_Weston Exp Jeff_Weston $

   $Log: editor.h $
   Revision 1.15  1997/09/02 07:36:36  Jeff_Weston
   - Added declarations for the new cursor bitmap.

   Revision 1.14  1997/08/23 01:03:31  Jeff_Weston
   - Added declarations for scrolling buttons and functions.
   - Added declarations for UNUSED map tiles.

   Revision 1.13  1997/08/22 23:45:17  Jeff_Weston
   - Removed integer constant associated with the number of land tiles.
   - Added a vector implementation of the land tile array.

   Revision 1.12  1997/08/22 05:55:53  Jeff_Weston
   - Move all constant strings into the header file.

   Revision 1.11  1997/08/22 04:04:11  Jeff_Weston
   - Added declaration for loading files from external BIG file.

   Revision 1.10  1997/07/30 00:41:16  Jeff_Weston
   - Changed declarations for version 2 of digit bitmaps.
   - Added declarations for initial version of save/load button bitmaps.
   - Added declaration for option phase display.
   - Changed positions of various windows according to agreed to standard.
   - Added declarations for save and load functionality.

   Revision 1.9  1997/07/24 07:41:52  Jeff_Weston
   - Added declarations to support Digit Bitmaps.
   - Added declarations to support Location Display code.
   - Added declarations to support Phase Display code.

   Revision 1.8  1997/07/23 22:31:28  Jeff_Weston
   - Added declaration support for 'Click and Drag' map editing.

   Revision 1.7  1997/07/23 06:59:55  Jeff_Weston
   - Added declarations to support scrolling code.

   Revision 1.6  1997/07/23 01:09:07  Jeff_Weston
   - Added declarations for map editor code.

   Revision 1.5  1997/07/22 23:27:33  Jeff_Weston
   - Added declaration support for Land Bitmap selection using the mouse.

   Revision 1.4  1997/07/22 21:59:05  Jeff_Weston
   - Added declaration for Select Bitmap.

   Revision 1.3  1997/07/22 07:42:04  Jeff_Weston
   - Removed Front and Back buffer messages.
   - Added iBuffer and iPause to class APPLICATION.
   - Changed LAND_BITMAPS to 12.

   Revision 1.2  1997/07/22 06:05:42  Jeff_Weston
   - Added declarations for Initial Version of Land Bitmaps
   - Added comment seperators.

   Revision 1.1  1997/07/22 00:15:35  Jeff_Weston
   Initial revision


*********************************************************************/

/****************** Constant Strings ********************************/
#define NAME         "MapEditor"
#define TITLE        "Map Editor for Dominate"

#define SAVE_ERROR   "Save Error"
#define SAVE_ERROR_1 "Unable to open map file for writing."
#define SAVE_ERROR_2 "Unable to save complete map. (Low on space?)"
#define SAVE_OK      "Save Success"
#define SAVE_OK_1    "The map has been saved."

#define LOAD_ERROR   "Load Error"
#define LOAD_ERROR_1 "Unable to open map file for reading."
#define LOAD_ERROR_2 "Unable to load map file information."
#define LOAD_ERROR_3 "This map file version is not supported."
#define LOAD_ERROR_4 "This map size is not supported."
#define LOAD_ERROR_5 "Map file is incorrect size (too small)."
#define LOAD_OK      "Load Success"
#define LOAD_OK_1    "The map has been loaded."

#define MAP_EXT      "map"

#define BIG_FILE     "DOMINATE.BIG"

#define LAND_BMP     "LAND%04i.BMP"
#define DIGIT_BMP    "DIGIT%i.BMP"
#define SELECT_BMP   "SELECT.BMP"
#define BUT_LOAD_BMP "BUT_LOAD.BMP"
#define BUT_SAVE_BMP "BUT_SAVE.BMP"
#define BUT_DOWN_BMP "BUT_DOWN.BMP"
#define BUT_UP_BMP   "BUT_UP.BMP"
#define CURSOR_BMP   "CURSOR.BMP"
#define UNUSED       "UNUSED.BMP"

#define NULL_STR     ""

/****************** Constant Integers *******************************/
#define PHASE_ON         0

#define TIMER_ID         1
#define TIMER_RATE       100

#define MODE_X           640
#define MODE_Y           480
#define MODE_B           8

#define LAND_X           64
#define LAND_Y           64

#define SELECT_X         66
#define SELECT_Y         66
#define SELECT_WIN_X     2
#define SELECT_WIN_Y     6
#define SELECT_DISP_X    10
#define SELECT_DISP_Y    10

#define DIGIT_BITMAPS    10
#define DIGIT_X          19
#define DIGIT_Y          19
#define DIGIT_DISP_X     10
#define DIGIT_DISP_Y     451

#define LOAD_BUTTON_X    66
#define LOAD_BUTTON_Y    25
#define LOAD_DISP_X      10
#define LOAD_DISP_Y      420

#define SAVE_BUTTON_X    66
#define SAVE_BUTTON_Y    25
#define SAVE_DISP_X      76
#define SAVE_DISP_Y      420

#define DOWN_BUTTON_X    10
#define DOWN_BUTTON_Y    10
#define DOWN_DISP_X      143
#define DOWN_DISP_Y      396

#define UP_BUTTON_X      10
#define UP_BUTTON_Y      10
#define UP_DISP_X        143
#define UP_DISP_Y        10

#define CURSOR_X         19
#define CURSOR_Y         29

#define PHASE_DISP_X     150
#define PHASE_DISP_Y     0

#define MAP_VIEW_X       7
#define MAP_VIEW_Y       7
#define MAP_SIZE_X       100
#define MAP_SIZE_Y       100
#define MAP_VIEW_START_X 155
#define MAP_VIEW_START_Y 10

#define MAP_FILE_VERSION 1

#define CLICK_EVENTS     65
#define LOAD_EVENT       61
#define SAVE_EVENT       62
#define DOWN_EVENT       63
#define UP_EVENT         64

#define BUFFER           256
#define PATH_BUFFER      1024

/****************** #includes and #defines **************************/
#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <commdlg.h>
#include <directx/ddraw.h>
#include "ddutil.h"
#include "error.h"
#include "bigutil.h"
#include "vector.h"

/****************** Defintion of Function WndProc *******************/
LRESULT CALLBACK WndProc (HWND hWnd, UINT iMsg,
                           WPARAM wParam, LPARAM lParam);

/****************** Defintion of Struct CLICKEVENT ******************/
struct CLICKEVENT
{
  RECT rcClickRect;
  int  iFunction, iX, iY;
};

/****************** Definition of Class APPLICATION *****************/
class APPLICATION
{
  public :
    APPLICATION ();

    LRESULT CALLBACK WindProc (HWND hWnd, UINT iMsg,
                               WPARAM wParam, LPARAM lParam);
    int Run (HINSTANCE hInstance, int iCmdShow);

  private :
    HWND                        hWnd;
    WNDCLASSEX                  WndClass;
    HCURSOR                     hOldCursor;

    LPDIRECTDRAW                lpDD;
    LPDIRECTDRAWPALETTE         lpDDPalSystem, lpDDPalGame;
    LPDIRECTDRAWSURFACE         lpDDSPrimary, lpDDSBack, lpDDSSelect;
    LPDIRECTDRAWSURFACE         lpDDSLoadButton, lpDDSSaveButton;
    LPDIRECTDRAWSURFACE         lpDDSDownButton, lpDDSUpButton;
    LPDIRECTDRAWSURFACE         lpDDSCursor;
    LPDIRECTDRAWSURFACE         lpDDSDigit[DIGIT_BITMAPS];
    vector<LPDIRECTDRAWSURFACE> lpDDSLand;

    BOOL                        bActive, bLMButtonDown;
    RECT                        rcSelectRect[SELECT_X][SELECT_Y];
    RECT                        rcMapViewRect[MAP_VIEW_X][MAP_VIEW_Y];
    POINT                       ptSelect;
    CLICKEVENT                  ceClickEvents[CLICK_EVENTS];
    OPENFILENAME                ofn;

    char                        iMapTile[MAP_SIZE_X][MAP_SIZE_Y];
    int                         iMapCornerX, iMapCornerY, iMouseX, iMouseY;
    int                         iLastX, iLastY, iCurX, iCurY, iPhase;
    int                         iLandBitmaps, iCurRow, iSelectedTile;

    BOOL Init             (HINSTANCE hInstance, int iCmdShow);
    void CleanUp          ();

    void SelectionDisplay ();
    void MapViewDisplay   ();
    void DigitDisplay     ();
    void PhaseDisplay     ();
    void ButtonDisplay    ();
    void CursorDisplay    ();
    void UpdateScreen     ();

    void SaveMap ();
    void LoadMap ();
    void ScrollD ();
    void ScrollU ();

    void Click1           (int iX, int iY);
    void Click2           (int iX, int iY);
    void Click3           (int iX, int iY);

    void Timer            ();
    void KeyDown          (WPARAM wParam);
    void LButtonDown      (int iX, int iY, int iMouseMove);
    void MouseMove        (LPARAM lParam);
};
